----------------------------------------------------------------------------
--                                  M2OS
--
--                           Copyright (C) 2019
--                    Universidad de Cantabria, SPAIN
--
--  Author: Mario Aldea Rivas (aldeam@unican.es)
--
--  This file is part of M2OS.
--
--  M2OS is free software: you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation, either version 3 of the License, or
--  (at your option) any later version.
--
--  M2OS is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with M2OS.  If not, see <https://www.gnu.org/licenses/>.
----------------------------------------------------------------------------
with M2.Queue;
with M2.Debug;
with M2.Direct_IO;

package body M2.Kernel.Ready_Queue is

   --  pragma No_Elaboration_Code_All;

   use type HAL.HWTime, TCBs.TCB_Ac;

   -----------------------------
   -- Suspended Threads Queue --
   -----------------------------

   function Later_Activation_Than (Left, Right : access TCBs.TCB)
                                   return Boolean is
   begin
      return Left.Activation_Time > Right.Activation_Time;
   end Later_Activation_Than;

   package Suspended_Threads_Queue is
     new M2.Queue (TCBs.TCB, Later_Activation_Than'Access);

   SQ : aliased Suspended_Threads_Queue.Queue := null;

   -------------------------
   -- Ready Threads Queue --
   -------------------------

   function Lower_Prio_Than (Left, Right : access TCBs.TCB)
                             return Boolean is
   begin
      return Left.Priority < Right.Priority;
   end Lower_Prio_Than;

   package Ready_Threads_Queue is
     new M2.Queue (TCBs.TCB, Lower_Prio_Than'Access);

   RQ : aliased Ready_Threads_Queue.Queue := null;

   -------------------------
   -- Head_Of_Ready_Queue --
   -------------------------

   function Head_Of_Ready_Queue return TCBs.TCB_Ac is
   begin
      pragma Assert (Ready_Threads_Queue.Head (RQ'Access) = null or else
                     Ready_Threads_Queue.Head (RQ'Access).Status = Ready);

      return Ready_Threads_Queue.Head (RQ'Access);
   end Head_Of_Ready_Queue;

   --------------------------
   -- Enqueue_Ready_Thread --
   --------------------------

   procedure Enqueue_Ready_Thread (Th_Ac : TCBs.TCB_Ac) is
   begin
      pragma Assert (Th_Ac.Status = Ready);

      Ready_Threads_Queue.Enqueue (Th_Ac, RQ'Access);
   end Enqueue_Ready_Thread;

   ---------------------------------
   -- Dequeue_Head_Of_Ready_Queue --
   ---------------------------------

   procedure Dequeue_Head_Of_Ready_Queue is
   begin
      pragma Assert (Ready_Threads_Queue.Head (RQ'Access) /= null and then
                     Ready_Threads_Queue.Head (RQ'Access).Status = Ready);

      Ready_Threads_Queue.Dequeue_Head (RQ'Access);
   end Dequeue_Head_Of_Ready_Queue;

   ------------------------------
   -- Enqueue_Suspended_Thread --
   ------------------------------

   procedure Enqueue_Suspended_Thread (Th_Ac : TCBs.TCB_Ac) is
   begin
      pragma Assert (Th_Ac.Status = Suspended);

      Suspended_Threads_Queue.Enqueue (Th_Ac, SQ'Access);
   end Enqueue_Suspended_Thread;

   --------------------------------
   -- Activate_Suspended_Threads --
   --------------------------------

   procedure Activate_Suspended_Threads is
      use type HAL.HWTime;

      Now : constant HAL.HWTime := HAL.Get_HWTime;
      Th_Ac : access TCBs.TCB;
   begin
      loop
         pragma Assert (HAL.Are_Interrupts_Enabled);

         Th_Ac := Suspended_Threads_Queue.Head (SQ'Access);

         exit when Th_Ac = null or else Th_Ac.Activation_Time > Now;
         pragma Assert (Th_Ac.Status = Suspended);

         Suspended_Threads_Queue.Dequeue_Head (SQ'Access);

         Th_Ac.Status := Ready;
         Ready_Threads_Queue.Enqueue (Th_Ac, RQ'Access);
      end loop;
   end Activate_Suspended_Threads;

   ----------
   -- Show --
   ----------

   procedure Show (DBG_Level : M2.Trace_Levels) is
      Th_Ac : TCBs.TCB_Ac;
   begin
      if not M2.Trace_Enabled (DBG_Level) then
         return;
      end if;

      Direct_IO.Put ("RQ:");
      Th_Ac := Ready_Threads_Queue.Head (RQ'Access);
      while Th_Ac /= null loop
         TCBs.Show (Th_Ac.all);
         Th_Ac := Th_Ac.Next;
      end loop;

      Direct_IO.Put ("SQ:");
      Th_Ac := Suspended_Threads_Queue.Head (SQ'Access);
      while Th_Ac /= null loop
         M2.Direct_IO.Put ("(");
         M2.Direct_IO.Put (Integer (Th_Ac.Id));
         M2.Direct_IO.Put (",");
         M2.Direct_IO.Put (Integer (Th_Ac.Activation_Time));
         M2.Direct_IO.Put (")");
         Th_Ac := Th_Ac.Next;
      end loop;
      Direct_IO.Put (":");
   end Show;

end M2.Kernel.Ready_Queue;
