----------------------------------------------------------------------------
--                                  M2OS
--
--                           Copyright (C) 2019
--                    Universidad de Cantabria, SPAIN
--
--  Author: Mario Aldea Rivas (aldeam@unican.es)
--
--  This file is part of M2OS.
--
--  M2OS is free software: you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation, either version 3 of the License, or
--  (at your option) any later version.
--
--  M2OS is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with M2OS.  If not, see <https://www.gnu.org/licenses/>.
----------------------------------------------------------------------------
with Ada.Unchecked_Conversion;
with System;
with Interfaces;

with M2.Kernel.Scheduler;

with M2.Direct_IO;

package body M2.Kernel.TCBs is

   pragma Compile_Time_Error (System.Thread_T_Size_In_Bytes * 8 /= TCB'Size,
                              "Unexpected size of TCB");

   use type Interfaces.Unsigned_16;

   --------------------
   -- Initialize_TCB --
   --------------------

   procedure Initialize_TCB
     (Th_Ac : TCB_Ac;
      Id : Kernel.Thread_Id;
      Priority : M2.Kernel.Priority;
      Init_Body_Ac : Kernel.Thread_Init_Body_Ac;
      Args : System.Address) is
   begin
      Th_Ac.Id := Id;
      Th_Ac.Context := Scheduler.Thread_Body_Wrapper'Access;
      Th_Ac.Priority  := Priority;
      Th_Ac.Status := Ready;
      Th_Ac.Init_Body_Ac := Init_Body_Ac;
      Th_Ac.Args :=  Args;
   end Initialize_TCB;

   --------
   -- Id --
   --------

   function Id (Th : in TCB)
                return Kernel.Thread_Id is
   begin
      return Th.Id;
   end Id;

   ----------------
   -- Set_Status --
   ----------------

   procedure Set_Status (Th : in out TCB;
                         St : Thread_Status) is
   begin
      Th.Status := St;
   end Set_Status;

   ------------------
   -- Set_Job_Body --
   ------------------

   procedure Set_Job_Body (Th : in out TCB;
                           Job_Body_Ac : Scheduler.Thread_Job_Body_Ac) is
   begin
      Th.Context := Job_Body_Ac;
   end Set_Job_Body;

   ------------
   -- Status --
   ------------

   function Status (Th : in TCB)
                    return Thread_Status is
   begin
      return Th.Status;
   end Status;

   -------------
   -- Context --
   -------------

   function Context (Th : in TCB)
                     return HAL.Thread_Context is
   begin
      return Th.Context;
   end Context;

   --------------
   -- Priority --
   --------------

   function Priority (Th : in TCB)
                      return Kernel.Priority is
   begin
      return Th.Priority;
   end Priority;

   ----------
   -- Show --
   ----------

   procedure Show  (Th : in TCB; Level : M2.Trace_Levels := M2.Trace) is
      use type M2.Kernel.Thread_Status;
   begin
      if not M2.Trace_Enabled (Level) then
         return;
      end if;

      M2.Direct_IO.Put ("(");
      M2.Direct_IO.Put (Integer (Th.Id));
      M2.Direct_IO.Put (",");
      M2.Direct_IO.Put (Integer (Th.Priority));
      M2.Direct_IO.Put (",");
      case Th.Status is
         when Ready =>
            M2.Direct_IO.Put ("R)");
         when Blocked =>
            M2.Direct_IO.Put ("B)");
         when Suspended =>
            M2.Direct_IO.Put ("S)");
      end case;
   end Show;

end M2.Kernel.TCBs;
