----------------------------------------------------------------------------
--                                  M2OS
--
--                           Copyright (C) 2019
--                    Universidad de Cantabria, SPAIN
--
--  Author: Mario Aldea Rivas (aldeam@unican.es)
--
--  This file is part of M2OS.
--
--  M2OS is free software: you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation, either version 3 of the License, or
--  (at your option) any later version.
--
--  M2OS is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with M2OS.  If not, see <https://www.gnu.org/licenses/>.
----------------------------------------------------------------------------
with Ada.Unchecked_Conversion;

with M2.Queue;
with M2.Queue_Implementation;

package body M2.Kernel.Timing_Events is
   package QImpl renames M2.Queue_Implementation;
   use type HAL.HWTime;

   pragma Compile_Time_Error
     (Timing_Event'Size /= System.Timing_Event_Size_In_Bytes * 8,
      "Unexpected size of Timing_Event");

   -----------------------
   -- Local subprograms --
   -----------------------

   procedure Cancel_Handler (Event_Ac : Timing_Event_Ac);

   function Mayor_Than (Left, Right : access Timing_Event) return Boolean is
   begin
      return Left.Expiration_Time > Right.Expiration_Time;
   end Mayor_Than;

   -----------
   -- Queue --
   -----------

   package Timing_Event_Queue is
     new M2.Queue (Timing_Event, Mayor_Than'Access);

   Q : aliased Timing_Event_Queue.Queue := null;

   -----------------
   -- Set_Handler --
   -----------------

   procedure Set_Handler (Event_Ac : Timing_Event_Ac;
                          At_Time : Time;
                          Handler : Timing_Event_Handler) is
   begin
      Cancel_Handler (Event_Ac);

      if Handler = null then
         return;
      end if;

      Event_Ac.Expiration_Time := At_Time;
      Event_Ac.Handler := Handler;
      Timing_Event_Queue.Enqueue (Event_Ac, Q'Access);
   end Set_Handler;

   --------------------------
   -- Set_Handler_Relative --
   --------------------------

   --  Not allowed in Ravenscar (pragma Restrictions No_Relative_Delay)

--     procedure Set_Handler_Relative (Event_Ac : Timing_Event_Ac;
--                                     In_Time : Time;
--                                     Handler : Timing_Event_Handler) is
--     begin
--        Set_Handler (Event_Ac, In_Time + HAL.Get_HWTime, Handler);
--     end Set_Handler_Relative;

   ---------------------
   -- Current_Handler --
   ---------------------

   function Current_Handler (Event : Timing_Event)
                             return Timing_Event_Handler is
   begin
      return Event.Handler;
   end Current_Handler;

   -----------------
   -- Clear_Event --
   -----------------

   procedure Clear_Event (Event_Ac : Timing_Event_Ac) is
   begin
      Event_Ac.Expiration_Time := HAL.HWTime'First;
      Event_Ac.Handler := null;
   end Clear_Event;

   --------------------
   -- Cancel_Handler --
   --------------------

   procedure Cancel_Handler (Event_Ac : Timing_Event_Ac) is
   begin
      if Event_Ac.Handler /= null then
         if M2.Allow_Timing_Events_Cancellation'First then
            Timing_Event_Queue.Dequeue (Event_Ac, Q'Access);
         else
            raise Program_Error;
         end if;
      end if;

      Clear_Event (Event_Ac);
   end Cancel_Handler;

   --------------------
   -- Cancel_Handler --
   --------------------

   procedure Cancel_Handler (Event_Ac : Timing_Event_Ac;
                             Cancelled : out Boolean_C) is
   begin
      Cancelled := Boolean_C (Event_Ac.Handler /= null);
      Cancel_Handler (Event_Ac);
   end Cancel_Handler;

   -------------------
   -- Time_Of_Event --
   -------------------

   function Time_Of_Event (Event : Timing_Event) return Time is
     (Event.Expiration_Time);

   ----------------
   -- Queue_Head --
   ----------------

   function Queue_Head return Timing_Event_Ac is
   begin
      return Timing_Event_Queue.Head (Q'Access);
   end Queue_Head;

   ------------------------
   -- Queue_Dequeue_Head --
   ------------------------

   procedure Queue_Dequeue_Head is
   begin
      Timing_Event_Queue.Dequeue_Head (Q'Access);
   end Queue_Dequeue_Head;

end M2.Kernel.Timing_Events;
