----------------------------------------------------------------------------
--                                  M2OS
--
--                           Copyright (C) 2019
--                    Universidad de Cantabria, SPAIN
--
--  Author: Mario Aldea Rivas (aldeam@unican.es)
--
--  This file is part of M2OS.
--
--  M2OS is free software: you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation, either version 3 of the License, or
--  (at your option) any later version.
--
--  M2OS is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with M2OS.  If not, see <https://www.gnu.org/licenses/>.
----------------------------------------------------------------------------

--  pragma Warnings (Off);
--  with System.OS_Interface;
--  pragma Warnings (On);
#if M2_TARGET = "arduino_uno" then
  with M2.HAL.Timer;
#end if;
with M2.Direct_IO;
with M2.HAL;

package body Measurements is
--     package OSI renames System.OS_Interface;
   package DIO renames M2.Direct_IO;

#if M2_TARGET = "arduino_uno" then
   package TIMER renames M2.HAL.Timer;

   type Time_Short is mod 250;
#end if;

#if M2_TARGET = "arduino_uno" then
   Us_Per_Tick : constant array (Measurement_Types) of Time :=
     (Long => 1E6 / M2.HAL.HWTime_Hz, Short => 4);

#elsif M2_TARGET = "stm32f4" or M2_TARGET = "epiphany" or M2_TARGET = "avr_iot" then
   Us_Per_Tick : constant array (Measurement_Types) of Time :=
     (Long => 1E6 / M2.HAL.HWTime_Hz);
#else
   pragma Compile_Time_Error (True, "Unexpected architecture");
#end if;

   ------------------
   -- Measurements --
   ------------------

   Num_Of_Measurements : Natural := 0;

   Total_Measurements_Time : Time := 0;

   Measurement_Start_Time : Time := 0;
   Measurement_End_Time : Time := 0;

   Measure_Time_Offset : Time := 0;

   Measurement_Type : Measurement_Types;

   Initialized : Boolean := False;

   ---------------------
   -- Time_Unit_In_Us --
   ---------------------

   function Time_Unit_In_Us return Time is
   begin
      pragma Assert (Initialized);

      return Us_Per_Tick (Measurement_Type);
   end Time_Unit_In_Us;

   ----------
   -- Init --
   ----------

   procedure Init (Measurement_Type : Measurement_Types) is
      Num_Of_Measures : constant := 1000;
   begin
      pragma Assert (not Initialized);

      Initialized := True;
      Measurements.Measurement_Type := Measurement_Type;

      --  Get Measure_Time_Offset

      for I in 1 .. Num_Of_Measures loop
         Start_Measurement;
         End_Measurement;
      end loop;

      Measure_Time_Offset := Total_Measurements_Time / Num_Of_Measures;

      --  Reset values to be ready for the "real" measurements

      Num_Of_Measurements := 0;
      Total_Measurements_Time := 0;
   end Init;

   -----------
   -- Reset --
   -----------

   procedure Reset is
   begin
      pragma Assert (Initialized);

      Num_Of_Measurements := 0;
      Total_Measurements_Time := 0;
   end Reset;

   --------------
   -- Get_Time --
   --------------

   function Get_Time return Time;

   function Get_Time return Time is
   begin
      case Measurement_Type is
         when Long =>
            return Time (M2.HAL.Get_HWTime);
#if M2_TARGET = "arduino_uno" then
         when Short =>
            return Time (TIMER.Get_Counter);
#end if;
      end case;
   end Get_Time;


   -----------------------
   -- Start_Measurement --
   -----------------------

   procedure Start_Measurement is
   begin
      pragma Assert (Initialized);

      Measurement_Start_Time := Get_Time;
   end Start_Measurement;

   ---------------------
   -- End_Measurement --
   ---------------------

   procedure End_Measurement is
   begin
      pragma Assert (Initialized);

      case Measurement_Type is
         when Long =>
            Total_Measurements_Time := Total_Measurements_Time +
              Get_Time - Measurement_Start_Time;
#if M2_TARGET = "arduino_uno" then
         when Short =>
            Total_Measurements_Time := Total_Measurements_Time +
              Time (Time_Short (Get_Time) -
                        Time_Short (Measurement_Start_Time));
#end if;
      end case;

      Num_Of_Measurements := Num_Of_Measurements + 1;
   end End_Measurement;

   ----------------------
   -- Measurement_Time --
   ----------------------

   function Measurement_Time return Time is
   begin
      return Total_Measurements_Time;
   end Measurement_Time;

   -----------------------------
   -- Measurement_Avg_Time_Us --
   -----------------------------

   function Measurement_Avg_Time_Us return Time is
   begin
      return (Total_Measurements_Time -
                Measure_Time_Offset * Time (Num_Of_Measurements))
        * Us_Per_Tick (Measurement_Type)
        / Time (Num_Of_Measurements);
   end Measurement_Avg_Time_Us;

   ------------
   -- Finish --
   ------------

   procedure Finish is
      pragma Suppress (Overflow_Check);
   begin
      pragma Assert (Initialized);

      DIO.Put ("Total:");
      DIO.Put (Integer (Total_Measurements_Time));
      DIO.Put (" N:");
      DIO.Put (Num_Of_Measurements);
      DIO.Put (" Off:");
      DIO.Put (Integer (Measure_Time_Offset));
#if M2_TARGET = "arduino_uno" then
      if Measurement_Type = Short then
         DIO.Put (" Avg(us):");
         DIO.Put (Integer (Measurement_Avg_Time_Us));
      end if;
#end if;
      DIO.New_Line;

      Tests_Reports.Test_OK;
   end Finish;

end Measurements;
